# archive.tcl - The Tape Interface
#
# Copyright (c) 1997, 1998, 1999 Mark Black
#
#
# This file contains:
#   tapeText    - Fill the text window with data
#   canBACKUP   - Draw the canvas for the backup tools
#   canRESTORE  - Draw the canvas for the restore tools
#   canTSCHED   - Draw the canvas for the tape scheduling tools

#   getTapeHost - Returns a list of tape servers
#   newTape     - Add a new backup tape to the tape server



#
# -- Tape Specific Procedure to read data from raw data and fill list window --
#    w = window to display it in
proc tapeText { data w type } {
    global currenthost
    global tapeserv
    . config -cursor watch
    $w config -cursor watch
    $w configure -state normal
    $w delete 0.0 end

    # Backup
    set tabbing(121) {6 16 24 }
    set toplabel(121) "ID    Name            Contents"
    # Backup Group
    set tabbing(123) {7 }
    set toplabel(123) "GID    Name"
    # Backup Job
    # job#:group#:retain:drive#:pool#:Descriptor
    set tabbing(125) {6 8 }
    set toplabel(125) "Job   Group   Name"
    # Backup Devices
    set tabbing(126) {4 8 }
    set toplabel(126) "ID  Slots   Name"
    # Media Maanagement
    set tabbing(127) {8 16 6 10 }
    set toplabel(127) "TapeID  Name    Vol#  DD/MM/YY  Contents"
    # Restore 
    set tabbing(131) {6 16 24 }
    set toplabel(131) "ID    Name            Contents"

    set bold "-background #c0c000 -relief raised -borderwidth 1"
    set normal "-background {} -relief flat"
    set select "-underline 1"
    set tplabel $w
    append tplabel "label"
    $tplabel configure -state normal
    $tplabel delete 0.0 end
    $tplabel insert end $toplabel($type)
    $tplabel configure -state disabled

    if { $type == 121 } {
	# ------------------------------  Backup  --------------------------------
	for { set position 0 } { $position < [llength $data] } { incr position 1 } {
	    set tmp [lindex $data $position ]
	    set tmp3 [split $tmp {;} ]
#	    puts stdout $tmp3
	    set tmp4 [lindex $tmp3 1 ]
	    set outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 0] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }
	    set tmp4 [lindex $tmp3 6 ]
	    append outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 1] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }

	    set tmp4 [string trim [lrange $tmp3 11 end ]]
	    append outline $tmp4
	    $w insert end "$outline\n" l$position
	    $w tag bind l$position <Any-Enter> "$w tag configure l$position $bold"
	    $w tag bind l$position <Any-Leave> "$w tag configure l$position $normal"
	    $w tag bind l$position <Button-1> "Selecter $w l$position"
	    $w tag bind l$position <Double-1>  "fileSelect $currenthost [lindex $tmp3 1 ] 0" 
	}
	$w configure -state disabled
    } elseif { $type == 123 } {
	# ------------------------------  Backup Group --------------------------------
	for { set position 0 } { $position < [llength $data] } { incr position 1 } {
	    set tmp [lindex $data $position ]
	    set tmp3 [split $tmp {:} ]
	    set tmp4 [lindex $tmp3 0 ]
	    set outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 0] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }

	    set tmp4 [string trim [lindex $tmp3 2]]
	    append outline $tmp4
	    $w insert end "$outline\n" l$position
	    $w tag bind l$position <Any-Enter> "$w tag configure l$position $bold"
	    $w tag bind l$position <Any-Leave> "$w tag configure l$position $normal"
	    $w tag bind l$position <Button-1> "Selecter $w l$position"
	    $w tag bind l$position <Double-1>  "tapeGroup $currenthost [lindex $tmp3 0 ] $tmp" 
	    $w tag configure l$position -foreground #000000
	}
	$w configure -state disabled
    } elseif { $type == 125 } {
	# ------------------------------  Backup Job --------------------------------
	for { set position 0 } { $position < [llength $data] } { incr position 1 } {
	    set tmp [lindex $data $position ]
	    set tmp3 [split $tmp {:} ]
	    puts stdout $tmp3
	    set tmp4 [lindex $tmp3 0 ]
	    set outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 0] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }
	    set tmp4 [lindex $tmp3 1 ]
	    append outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 1] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }

	    set tmp4 [string trim [lindex $tmp3 6 ]]
	    append outline $tmp4
	    $w insert end "$outline\n" l$position
	    $w tag bind l$position <Any-Enter> "$w tag configure l$position $bold"
	    $w tag bind l$position <Any-Leave> "$w tag configure l$position $normal"
	    $w tag bind l$position <Button-1> "Selecter $w l$position"
	    $w tag bind l$position <Double-1>  "tapeJobs $currenthost [lindex $tmp3 0 ] $tmp" 
	    $w tag configure l$position -foreground #000000
	}
	$w configure -state disabled
    } elseif { $type == 126 } {
	# ------------------------------  Backup Devices --------------------------------
	for { set position 0 } { $position < [llength $data] } { incr position 1 } {
	    set tmp [lindex $data $position ]
	    set tmp3 [split $tmp {:} ]
	    puts stdout $tmp3
	    set tmp4 [lindex $tmp3 0 ]
	    set outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 0] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }
	    set tmp4 [lindex $tmp3 4 ]
	    append outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 1] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }

	    set tmp4 [string trim [lindex $tmp3 1]]
	    append outline $tmp4
	    $w insert end "$outline\n" l$position
	    $w tag bind l$position <Any-Enter> "$w tag configure l$position $bold"
	    $w tag bind l$position <Any-Leave> "$w tag configure l$position $normal"
	    $w tag bind l$position <Button-1> "Selecter $w l$position"
	    $w tag bind l$position <Double-1>  "tapeDevs $currenthost [lindex $tmp3 0 ]" 
	    $w tag configure l$position -foreground #000000
	}
	$w configure -state disabled
    } elseif { $type == 127 } {
	# ------------------------------  Tape lists  --------------------------------
	# The data format of the tape lists is determined by the getTapeInfo function
	# eg:   2:First Tape to write                -  TapeID:Tape name
	#       0:27/12/98:1:Backup Job to tape 3    -  Volume#:Backup date:Expired:Job description
	#
	set pzn 0
	for { set position 0 } { $position < [llength $data] } { incr position 1 } {
	    set tmp [lindex $data $position ]
	    # We will need a simple state machine to parse the data correctly
	    set tmp3 [split $tmp {:} ]
	    puts stdout $tmp3
	    if { [llength $tmp3 ] == 2 } {
		# This is a new tape
		incr pzn 1
		set tmp4 [lindex $tmp3 0 ]
		set outline $tmp4
		for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 0] -[string length $tmp4 ]) } { incr cnt 1 } {
		    append outline " "
		}
		set tmp4 [lindex $tmp3 1 ]
		append outline $tmp4
		$w insert end "$outline\n" l$pzn
		$w tag bind l$pzn <Any-Enter> "$w tag configure l$pzn $bold"
		$w tag bind l$pzn <Any-Leave> "$w tag configure l$pzn $normal"
		$w tag bind l$pzn <Button-1> "Selecter $w l$pzn"
		#   $w tag bind l$pzn <Double-1>  "tapeDevs $currenthost [lindex $tmp3 0 ]" 
	    } else {
		# It's a tape volume
		set outline ""
		puts stdout "Line = $tmp3"
		for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 1]) } { incr cnt 1 } {
		    append outline " "
		}
		set tmp4 [lindex $tmp3 0 ]
		append outline $tmp4
		for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 2] -[string length $tmp4 ]) } { incr cnt 1 } {
		    append outline " "
		}
		set tmp4 [lindex $tmp3 1 ]
		append outline $tmp4
		for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 3] -[string length $tmp4 ]) } { incr cnt 1 } {
		    append outline " "
		}
		set tmp4 [lindex $tmp3 3 ]
		append outline $tmp4
		$w insert end "$outline\n" l$pzn
		$w tag bind l$pzn <Any-Enter> "$w tag configure l$pzn $bold"
		$w tag bind l$pzn <Any-Leave> "$w tag configure l$pzn $normal"
		$w tag bind l$pzn <Button-1> "Selecter $w l$pzn"
		if { [lindex $tmp3 2 ] == 1 } {
		    $w tag configure l$pzn -foreground #a00000
		} else {
		    $w tag configure l$pzn -foreground #000000
		}
		    
	    }

	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 1] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }

	}
	$w configure -state disabled
    } elseif { $type == 131 } {
	# ------------------------------  Restore  --------------------------------
	for { set position 0 } { $position < [llength $data] } { incr position 1 } {
	    set tmp [lindex $data $position ]
	    set tmp3 [split $tmp {;} ]
	    puts stdout $tmp3
	    set tmp4 [lindex $tmp3 1 ]
	    set outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 0] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }
	    set tmp4 [lindex $tmp3 6 ]
	    append outline $tmp4
	    for { set cnt 0 } { $cnt < ([lindex $tabbing($type) 1] -[string length $tmp4 ]) } { incr cnt 1 } {
		append outline " "
	    }

	    set tmp4 [string trim [lrange $tmp3 11 end ]]
	    append outline $tmp4
	    $w insert end "$outline\n" l$position
	    $w tag bind l$position <Any-Enter> "$w tag configure l$position $bold"
	    $w tag bind l$position <Any-Leave> "$w tag configure l$position $normal"
	    $w tag bind l$position <Button-1> "Selecter $w l$position"
	    $w tag bind l$position <Double-1> \
		    "resSelect $currenthost [lindex $tmp3 1 ] \"[lindex $tmp3 6 ]\" \"[string trim [lrange $tmp3 11 end ]]\"" 
	}
	$w configure -state disabled
    } 
}


# getDir  - Read a directory from the remote machine and
#           populate the listbox $lb.  If $dir is NOT "/"
#           then include ".." in the listbox.
proc getDir { host dir lb } {
    global errno
    $lb delete 0 end
 
    set cmd "TAPE get dir $dir"
    puts stdout "Cmd = $cmd"
    set retval [queryHost $host $cmd 0 ]
    if { $errno != 0 } {
	error 402
	puts stdout "ERROR:  Unable to retrieve directory information from $host"
    }
    
    if { $dir != "/" } {
	$lb insert end ".."
    }
    foreach e $retval {
	$lb insert end $e
    }
}




#
# canBACKUP  - Fill canvas with tape backup icons
#
proc canBACKUP {} {
    # ---------------------  Tape Backup Window  ---------------------
    global currenthost
    global level
    global browse
    global tapeserv
    .tp.title configure -text "Host($currenthost) --> Config --> Archives --> Backup "
    set level 120
    # Enable the accelerators
    bind .tp.host.canvas <Button-3> "rclick .tp.host.canvas"
    # Put in back button
    set fig(121) [ image create photo -file "../lib/new.gif" ]
    set fig(122) [ image create photo -file "../lib/tapehost.gif" ]
    set fig(123) [ image create photo -file "../lib/tapegrp.gif" ]
    set fig(125) [ image create photo -file "../lib/tapejob.gif" ]
    set fig(126) [ image create photo -file "../lib/tapedevs.gif" ]
    set fig(127) [ image create photo -file "../lib/tapes.gif" ]
    set fig(140) [ image create photo -file "../lib/sched.gif" ]

    set fig(0) [ image create photo -file "../lib/back.gif" ]
    set startX 40
    set startY 40
    set icon_size 48
    set incX 70
    set incY 80
    set cntx 0

    set node [ .tp.host.canvas create image $startX $startY -image $fig(0) -tags "<-Back" ]
    .tp.host.canvas bind "<-Back" <Double-1> "repack -60"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "<- Back" -tags nameLabel \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1

    # Draw an icon for the Tape drives
    set node [ .tp.host.canvas create image $startX $startY -image $fig(126) -tags tdevs ]
    .tp.host.canvas bind tdevs <Double-1> "set browse 126 ;repack 126"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "Backup\nDevices" -tags tdevs \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1

    # Draw an icon for the backup host groups
    set node [ .tp.host.canvas create image $startX $startY -image $fig(123) -tags tgrp ]
    .tp.host.canvas bind tgrp <Double-1> "set browse 123 ;repack 123"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "Backup\nGroup" -tags tgrp \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1

    # Draw an icon for the backup jobids
    set node [ .tp.host.canvas create image $startX $startY -image $fig(125) -tags tjob ]
    .tp.host.canvas bind tjob <Double-1> "set browse 125 ;repack 125"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "Backup\nJob" -tags tjob \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1

    # Draw an icon for the Tapes
    set node [ .tp.host.canvas create image $startX $startY -image $fig(127) -tags tapes ]
    .tp.host.canvas bind tapes <Double-1> "set browse 127 ;repack 127"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "Media\nManagement" -tags tapes \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1

    # Draw an icon for the backup schedule
    set node [ .tp.host.canvas create image $startX $startY -image $fig(140) -tags ttime ]
    .tp.host.canvas bind ttime <Double-1> "set browse 140 ;repack 140"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "Backup\nSchedule" -tags ttime \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1

}


#
# canRESTORE  - Fill canvas with tape restore icons
#
proc canRESTORE {} {
    # ---------------------  Tape Restore Window  ---------------------
    global currenthost
    global level
    global browse
    global tapeserv
    .tp.title configure -text "Host($currenthost) --> Config --> Archives --> Restore "
    set level 130
    # Enable the accelerators
    bind .tp.host.canvas <Button-3> "rclick .tp.host.canvas"
    # Put in back button
    set fig(132) [ image create photo -file "../lib/tapehost.gif" ]
    set fig(0) [ image create photo -file "../lib/back.gif" ]
    set startX 40
    set startY 40
    set icon_size 48
    set incX 70
    set incY 80
    set cntx 0
    set node [ .tp.host.canvas create image $startX $startY -image $fig(0) -tags "<-Back" ]
    .tp.host.canvas bind "<-Back" <Double-1> "repack -60"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "<- Back" -tags nameLabel \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1
    
    # Get a list of all the groups, and compile a list of all the hosts
    set rlist {}
    set rdata [queryHost $currenthost "TAPE get group" 0 ]
    foreach l $rdata {
	set tmp [lindex [split $l ":" ] 1 ]
	set tmp2 [split $tmp "," ]
	append rlist "$tmp2 "
    }

    # Look for doubles
    set cnt 0
    set hlist {}
    for { set pn 0 } { $pn < [llength $rlist] } { incr pn 1 } {
	set h [lindex $rlist $pn ]
	set flag 0
	for { set hpn 0 } { $hpn < [llength $hlist] } { incr hpn 1 } {
	    if { [lindex $hlist $hpn ] == $h } {
		# Match!!
		set flag 1
	    }
	}
	if { $flag == 0 } {
	    append hlist "$h "
	}
    }

    # Draw icons for each Tape server
    foreach tapeS $hlist {
	set node [ .tp.host.canvas create image $startX $startY -image $fig(132) -tags $tapeS ]
	.tp.host.canvas bind $tapeS <Double-1> "set tapeserv $tapeS ;repack 131 "
	set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
		    -justify center -text "$tapeS" -tags nameLabel \
		    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
	incr startX $incX 
	if { $cntx == 8 } {
	    set startX 40
	    incr startY $incY 
	    incr cntx 1
	}
    }
}


#
# canTSCHED  - Tape scheduling window
#
proc canTSCHED {} {
# ---------------------  Tape Restore Window  ---------------------
    global currenthost
    global level
    global browse
    global tapeserv
    .tp.title configure -text "Host($currenthost) --> Config --> Archives --> Restore "
    set level 130
    # Enable the accelerators
    bind .tp.host.canvas <Button-3> "rclick .tp.host.canvas"
    # Put in back button
    set fig(132) [ image create photo -file "../lib/tapehost.gif" ]
    set fig(0) [ image create photo -file "../lib/back.gif" ]
    set startX 40
    set startY 40
    set icon_size 48
    set incX 70
    set incY 80
    set cntx 0
    set node [ .tp.host.canvas create image $startX $startY -image $fig(0) -tags "<-Back" ]
    .tp.host.canvas bind "<-Back" <Double-1> "repack -60"
    set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
	    -justify center -text "<- Back" -tags nameLabel \
	    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
    incr startX $incX 
    incr cntx 1
    
    # Draw icons for each Tape server
    foreach tapeS [ getTapeHost ] {
	set node [ .tp.host.canvas create image $startX $startY -image $fig(132) -tags $tapeS ]
	.tp.host.canvas bind $tapeS <Double-1> "set tapeserv $tapeS ;repack 131 "
	set nameLabel [ .tp.host.canvas create text $startX [expr $startY + 0.8 * $icon_size] \
		    -justify center -text "$tapeS" -tags nameLabel \
		    -font {-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*} ]
	incr startX $incX 
	if { $cntx == 8 } {
	    set startX 40
	    incr startY $incY 
	    incr cntx 1
	}
    }
}


#
# getTapeHost - Read the MAT hosts file and return a list
#               of all tape servers.
#
proc getTapeHost {} {
    global active_hosts
    set retval {}
    foreach l $active_hosts {
	set tmp [ lindex [split $l ":" ] 3 ]
	foreach e [ split $tmp "," ] {
	    if { $e == "TAPE" } {
		# It's a tape host
		lappend retval [ lindex [split $l ":" ] 0 ]
	    }
	}
    }
    return $retval
}


#
# tapeJobs - Manage the tape job requests.  This GUI allows the user to set the
#            the following:   Job ID#   - Used to schedule the job
#                             Backup Gruop  - The hosts to backup
#                             Retention     - How long to hold a tape
#                             Drive         - Desired backup device
#                             Pool          - Pool of tapes available for the backup
#            Set jobnum to 9999 for new jobs
#
set sd {}
proc tapeJobs { tapeserv jobnum data args } {
    if { [string length $args ] != 0 } {
        append line " $args"
    }
    global errno
    global sd
    set sd "/"
    set host $tapeserv
    if { [winfo exists .tjob ] != 0 } {
	destroy .tjob
    }

    toplevel .tjob -class Dialog
    wm title .tjob "Backup Job"
    wm iconname .tjob "Backup Job"
    entry .tjob.hidden1 
    entry .tjob.hidden2
    .tjob.hidden1 insert end $tapeserv
    .tjob.hidden2 insert end $jobnum

    frame .tjob.f2 -bd 2 -relief "raised"
    label .tjob.f2.l1 -text "Backup Job Properties" 
    pack .tjob.f2.l1 -fill x

    frame .tjob.f2.f1 -bd 2 -relief "flat"
    label .tjob.f2.f1.l1 -text "Job #" -width 6
    entry .tjob.f2.f1.e1 -width 4
    label .tjob.f2.f1.l2 -text "Description:" -width 15
    entry .tjob.f2.f1.e2 -width 48
    pack .tjob.f2.f1.l1 .tjob.f2.f1.e1 .tjob.f2.f1.l2 .tjob.f2.f1.e2 -fill x -side left

    frame .tjob.f2.f2 -bd 2 -relief "flat"
    label .tjob.f2.f2.l1 -text "Backup Group" -width 15
    menubutton .tjob.f2.f2.b1 -text "..." -menu .tjob.f2.f2.b1.m -relief raised -width 3
    menu .tjob.f2.f2.b1.m -tearoff 0 
    set rdata [queryHost $tapeserv "TAPE get groups" 0 ]
    foreach l $rdata {
	set val [lindex [split $l ":"] 0]
	.tjob.f2.f2.b1.m add command -label $val -command ".tjob.f2.f2.e1 configure -state normal ; .tjob.f2.f2.e1 delete 0 end ; .tjob.f2.f2.e1 insert end \"$val\" ; .tjob.f2.f2.e1 configure -state disabled "
    }
    entry .tjob.f2.f2.e1 -width 7
    label .tjob.f2.f2.l2 -text "Retention (weeks):" -width 21
    menubutton .tjob.f2.f2.b2 -text "..." -menu .tjob.f2.f2.b2.m -relief raised -width 3
    menu .tjob.f2.f2.b2.m -tearoff 0 
    set vals { 1 2 4 8 12 16 26 52 }
    foreach val $vals {
	.tjob.f2.f2.b2.m add command -label $val -command ".tjob.f2.f2.e2 delete 0 end ; .tjob.f2.f2.e2 insert end \"$val\" "
    }
    entry .tjob.f2.f2.e2 -width 7
    pack .tjob.f2.f2.l1 .tjob.f2.f2.b1 .tjob.f2.f2.e1 .tjob.f2.f2.l2 .tjob.f2.f2.b2 .tjob.f2.f2.e2 -fill both -side left

    frame .tjob.f2.f4 -bd 2 -relief "flat"
    label .tjob.f2.f4.l1 -text "Backup Device:" -width 15
    menubutton .tjob.f2.f4.b1 -text "..." -menu .tjob.f2.f4.b1.m -relief raised -width 3
    menu .tjob.f2.f4.b1.m -tearoff 0 
    set vals {}
    set rdata [queryHost $tapeserv "TAPE get drives" 0 ]
    for { set p 0 } { $p < [llength $rdata] } { incr p 1 } {
	set tmp [lindex $rdata $p ]
	set t1 [lindex [split [lindex $rdata $p ] {:} ] 1 ]
	if { $t1 != {} } {
	    lappend vals $t1
	}
    }
    foreach val $vals {
	.tjob.f2.f4.b1.m add command -label $val -command ".tjob.f2.f4.e1 configure -state normal ; .tjob.f2.f4.e1 delete 0 end ; .tjob.f2.f4.e1 insert end \"$val\" ; .tjob.f2.f4.e1 configure -state disabled "
    }
    entry .tjob.f2.f4.e1 -width 16
    label .tjob.f2.f4.l2 -text "Backup Pool:" -width 14
    entry .tjob.f2.f4.e2 -width 7
    pack .tjob.f2.f4.l1 .tjob.f2.f4.b1 .tjob.f2.f4.e1 .tjob.f2.f4.l2 .tjob.f2.f4.e2 -fill x -side left

    pack .tjob.f2.f1 .tjob.f2.f2 .tjob.f2.f4 -side top -fill both -expand 1

    # -------------------   Browse Host and File Selection  ---------------------------
    frame .tjob.f5 -bd 2 -relief "raised"
    label .tjob.f5.l1 -text "Requested files to include in backup"

    frame .tjob.f5.f1 -bd 2 -relief "flat"
    label .tjob.f5.f1.l1 -text "Browse Host:" -width 13
    button .tjob.f5.f1.b1 -text "..." -padx 0 -command "FillGrpList .tjob $host .tjob.f2.f2.e1 .tjob.f5.f1.e1"
    entry .tjob.f5.f1.e1 -width 16
    .tjob.f5.f1.e1 delete 0 end
    .tjob.f5.f1.e1 insert end $tapeserv
    label .tjob.f5.f1.l2 -text "Current Dir:" -width 13
    entry .tjob.f5.f1.e2 -width 27
    pack .tjob.f5.f1.l1 .tjob.f5.f1.b1 .tjob.f5.f1.e1 .tjob.f5.f1.l2 .tjob.f5.f1.e2 -fill x -side left
    pack .tjob.f5.f1 -side top
 
    frame .tjob.f5.f2 -bd 2 -relief "flat"
    scrollbar .tjob.f5.f2.vscroll -orient vertical -command ".tjob.f5.f2.text yview"
    scrollbar .tjob.f5.f2.hscroll -orient horizontal -command ".tjob.f5.f2.text xview"
    listbox .tjob.f5.f2.text -selectforeground white -selectbackground #000080 \
	    -yscrollcommand {.tjob.f5.f2.vscroll set } \
	    -xscrollcommand {.tjob.f5.f2.hscroll set } -height 10 -width 32
    .tjob.f5.f2.text config -cursor hand2
    pack .tjob.f5.f2.vscroll -side left -fill y
    pack .tjob.f5.f2.hscroll -side bottom -fill x
    pack .tjob.f5.f2.text -side left -expand 1 -fill both

    frame .tjob.f5.f4 -bd 2 -relief "flat"
    button .tjob.f5.f4.b1 -text "Add" -command {
	if { [.tjob.f5.f2.text curselection ] != "" } {
	    .tjob.f5.f6.text insert end [string trimright "[.tjob.f5.f1.e2 get ][.tjob.f5.f2.text get [.tjob.f5.f2.text curselection ]]" {.} ]
	}
    }
    button .tjob.f5.f4.b2 -text "Remove" -command {
	if { [.tjob.f5.f6.text curselection ] != "" } {
	    .tjob.f5.f6.text delete [.tjob.f5.f6.text curselection ]
	}
    }
    pack .tjob.f5.f4.b1 .tjob.f5.f4.b2 -side top -fill both -anchor center 
    
    frame .tjob.f5.f6 -bd 2 -relief "flat"
    scrollbar .tjob.f5.f6.vscroll -orient vertical -command ".tjob.f5.f6.text yview" 
    scrollbar .tjob.f5.f6.hscroll -orient horizontal -command ".tjob.f5.f6.text xview"
    listbox .tjob.f5.f6.text -selectforeground white -selectbackground #000080 \
	    -yscrollcommand {.tjob.f5.f6.vscroll set } \
	    -xscrollcommand {.tjob.f5.f6.hscroll set } -height 10 -width 32
    .tjob.f5.f6.text config -cursor hand2
    pack .tjob.f5.f6.vscroll -side left -fill y
    pack .tjob.f5.f6.hscroll -side bottom -fill x
    pack .tjob.f5.f6.text -side left -expand 1 -fill both
    
    pack .tjob.f5.f2 .tjob.f5.f4 .tjob.f5.f6 -side left -fill both
    
    # -----------------------------------------------------------------------------------
    frame .tjob.f7 -bd 2 -relief "raised"
    button .tjob.f7.b1 -text "Apply" -foreground red -activeforeground #a00000 -command {
	if { [ .tjob.hidden2 get ] != 9999 } {
	    set cmd "TAPE mod jobids [ .tjob.hidden2 get ] "
	} else {
	    set cmd "TAPE add jobids "
	}
	append cmd "[ .tjob.f2.f1.e1 get ]:[ .tjob.f2.f2.e1 get ]:[ week2sec [ .tjob.f2.f2.e2 get ] ]:"
	# Will have to go and get the drive info from the machine
	set mdata [queryHost [ .tjob.hidden1 get ] "TAPE get drives" 0 ]
	foreach m $mdata {
	    if { [ .tjob.f2.f4.e1 get ] == [lindex [split $m ":"] 1] } {
		append cmd "[lindex [split $m ":"] 0]:"
	    }
	}
	append cmd "[ .tjob.f2.f4.e2 get ]:0:[ .tjob.f2.f1.e2 get ]"
	puts stdout "Cmd = $cmd"

	set returnval [queryHost [.tjob.hidden1 get] $cmd 1 ]
	if { $errno != 0 } {
	    error 2001
	    puts stdout "ERROR:  Unable to add new job to [.tjob.hidden1 get]"
	} else {
	    repack 125
	    destroy .tjob
	}
    }
    button .tjob.f7.b2 -text "Cancel" -foreground blue -activeforeground #000080 -command {
	destroy .tjob
    }
    button .tjob.f7.b3 -text "Help" -foreground #008000 -activeforeground #004000 -command {
	help tapejob
    }
    pack .tjob.f7.b1 .tjob.f7.b2 .tjob.f7.b3 -side left -fill x 
    
    pack .tjob.f2 .tjob.f5 .tjob.f7 -side top -fill both -expand 1

    # ------------ Bindings --------------------
    bind .tjob.f5.f2.text <Button-3> {
	.tjob.f5.f6.text insert end [string trimright "[.tjob.f5.f1.e2 get ][.tjob.f5.f2.text get @%x,%y ]" {.} ]
	add2backup [.tjob.f2.f1.e1 get ] [string trimright "[.tjob.f5.f1.e2 get ][.tjob.f5.f2.text get @%x,%y ]" {.} ]
    }
    bind .tjob.f5.f6.text <Button-3> {
	.tjob.f5.f6.text delete @%x,%y
	delbackup [.tjob.f2.f1.e1 get ] [string trimright "[.tjob.f5.f1.e2 get ][.tjob.f5.f2.text get @%x,%y ]" {.} ]
    }
    bind .tjob.f5.f6.text <Double-1> {
	if { [.tjob.f5.f6.text curselection ] != "" } {
	    .tjob.f5.f6.text delete [.tjob.f5.f6.text curselection ]
	}
    }
    bind .tjob.f5.f2.text <Double-1> {
	if { [.tjob.f5.f2.text curselection ] != {} } {
	    set t [.tjob.f5.f2.text get [.tjob.f5.f2.text curselection ] ]
	    if { $t == ".." } {
		# Need to go down a directory
		global sd
		set tmp [string trimright $sd {/} ]
		set ldir [string last {/} $tmp ]
		if { $ldir <= 0 } {
		    set sd "/"
		} else {
		    puts stdout "ldir = $ldir"
		    set tmp [string range $sd 0 $ldir ]
		    set sd $tmp
		    puts stdout "ldir = $ldir, sd = $sd"
		}
		getDir [.tjob.f5.f1.e1 get] $sd .tjob.f5.f2.text
	    } elseif { [string range $t end end ] == {/} } {
		global sd
		#set sd [string trimright [.tjob.f5.f2.text get [.tjob.f5.f2.text curselection ] ] {/} ]
		set sd "[.tjob.f5.f1.e2 get ][.tjob.f5.f2.text get [.tjob.f5.f2.text curselection ] ]"
		#set sd [.tjob.f5.f2.text get [.tjob.f5.f2.text curselection ] ]
		getDir [.tjob.f5.f1.e1 get] $sd .tjob.f5.f2.text
	    }
	}
	.tjob.f5.f1.e2 delete 0 end
	.tjob.f5.f1.e2 insert end "$sd"

    }


    # ---------- Fill boxes with data --------------------------
    global sd
    set sd "/"
    .tjob.f5.f1.e2 delete 0 end
    .tjob.f5.f1.e2 insert end "$sd"
    .tjob.f2.f1.e1 configure -state normal
    .tjob.f2.f1.e1 delete 0 end
    .tjob.f2.f2.e1 configure -state normal
    .tjob.f2.f2.e1 delete 0 end
    .tjob.f2.f2.e2 delete 0 end
    .tjob.f2.f4.e1 delete 0 end
    .tjob.f2.f4.e2 delete 0 end
    .tjob.f2.f1.e2 delete 0 end
    .tjob.f5.f6.text delete 0 end
    if { $jobnum != 9999 } {
	# Go get the jobdata from the tape server 
	set rdata [queryHost [ .tjob.hidden1 get ] "TAPE get jobids" 0 ]
	foreach l $rdata {
	    if { $jobnum == [lindex [split $l ":"] 0] } {
		set tmp [split $l ":"]
		puts stdout "Data = $tmp"
		.tjob.f2.f1.e1 insert end [lindex $tmp 0]
		.tjob.f2.f2.e1 insert end [lindex $tmp 1]
		.tjob.f2.f4.e2 insert end [lindex $tmp 4]
		.tjob.f2.f1.e2 insert end [lindex $tmp 6]
		# Populate the retention
		.tjob.f2.f2.e2 insert end [ sec2week [lindex $tmp 2] ]
		# Will have to go and get the group info from the machine
		set mdata [queryHost [ .tjob.hidden1 get ] "TAPE get drives" 0 ]
		foreach m $mdata {
		    if { [lindex $tmp 3] == [lindex [split $m ":"] 0] } {
			.tjob.f2.f4.e1 insert end [lindex [split $m ":"] 1]
		    }
		}
	    }
	}
	.tjob.f2.f1.e1 configure -state disabled
	.tjob.f2.f2.e1 configure -state disabled

	# ---- It's an old job, so populate the selected list
	set mdata [queryHost [ .tjob.hidden1 get ] "TAPE get backuplist [ .tjob.hidden2 get ]" 0 ]
	foreach m $mdata {
	    .tjob.f5.f6.text insert end $m
	}
    }
    getDir [.tjob.f5.f1.e1 get] $sd .tjob.f5.f2.text
}


#
# add2backup - Add an entry to the backup list
#
proc add2backup { jobnum entry args } {
    global currenthost
    global errno
    if { [string length $args ] != 0 } {
        append entry " $args"
    }
    if { ( $jobnum != {} ) & ( $entry != {} ) } {
	set mdata [queryHost $currenthost "TAPE add backuplist $jobnum $entry" 0 ]
	if { $errno != 0 } {
	    error 501
	    puts stdout "ERROR:  Can't update job backup list for job# $jobnum"
	}
    }
}
 

#
# delbackup - Delete an entry from the backup list
#
proc delbackup { jobnum entry args } {
    global currenthost
    global errno
    if { [string length $args ] != 0 } {
        append entry " $args"
    }
    if { ( $jobnum != {} ) & ( $entry != {} ) } {
	set mdata [queryHost $currenthost "TAPE del backuplist $jobnum $entry" 0 ]
	if { $errno != 0 } {
	    error 402
	    puts stdout "ERROR:  Can't update job backup list for job# $jobnum"
	}
    }
}


#
# week2sec - Convert the number of weeks to seconds ad return it
#
proc week2sec { week } {
    return [expr $week * 604800 ]
}


#
# sec2week - Convert the number of seconds to weeks and return it
#
proc sec2week { sec } {
    return [expr $sec / 604800 ]
}


#
# tapeGroup - Manage the tape host groups.  The group controls which
#             hosts are included in a backup job.
#
# USE: tapeGroup CurrentHost CurrentGroup SelectedData
#             CurrentHost = host to update
#             CurrentGroup = The current selected group
#                           If 9999 then it is a new entry
#             SelectedData = line of data to modify
# NOTE: Data must be seperated by a single space or a singel tab
#
proc tapeGroup { currenthost currentgroup line args } {
    if { [string length $args ] != 0 } {
        append line " $args"
    }
    global errno
    set host $currenthost
    if { [winfo exists .tgrp ] == 0 } {
	toplevel .tgrp -class Dialog
	wm title .tgrp "Backup Group Editor: $currenthost"
	wm iconname .tgrp "Backup Group"
	entry .tgrp.hidden1 
	entry .tgrp.hidden2
	# NOT GOOD
	.tgrp.hidden1 insert end $currenthost
	.tgrp.hidden2 insert end $currentgroup

	frame .tgrp.f1 -bd 2 -relief "raised"
	label .tgrp.f1.l1 -text "Group:" -width 6
	entry .tgrp.f1.e1 -width 24
	pack .tgrp.f1.l1 .tgrp.f1.e1 -fill both -side left -expand 1

	frame .tgrp.f2 -bd 2 -relief "raised"
	label .tgrp.f2.l1 -text "GID #:" -width 6
	entry .tgrp.f2.e1 -width 10
	pack .tgrp.f2.l1 .tgrp.f2.e1 -fill both -side left -expand 1

	frame .tgrp.f3 -bd 2 -relief "raised"
	label .tgrp.f3.l1 -text "Member Hosts" 
	pack .tgrp.f3.l1 -fill x
	frame .tgrp.f3.f1 -bd 2 -relief "flat"
	button .tgrp.f3.f1.b0 -text "..." -padx 0 -command "FillTapeList .tgrp $currenthost .tgrp.f3.f1.e1"
	entry .tgrp.f3.f1.e1 -width 10
	button .tgrp.f3.f1.b1 -text "Add" -command {
	    if { [.tgrp.f3.f1.e1 get] != "" } {
		.tgrp.f3.f2.text insert end [.tgrp.f3.f1.e1 get]
	    }
	    .tgrp.f3.f1.e1 delete 0 end
	}
	pack .tgrp.f3.f1.b0 .tgrp.f3.f1.e1 .tgrp.f3.f1.b1 -fill both -side left -expand 1
	frame .tgrp.f3.f2 -bd 2 -relief "flat"
	scrollbar .tgrp.f3.f2.vscroll -orient vertical -command ".tgrp.f3.f2.text yview" 
	listbox .tgrp.f3.f2.text -selectforeground white -selectbackground #000080 -yscrollcommand {.tgrp.f3.f2.vscroll set } -height 5 -width 10
	.tgrp.f3.f2.text config -cursor hand2
	pack .tgrp.f3.f2.vscroll -side left -fill y
	pack .tgrp.f3.f2.text -side left -expand 1 -fill both
	button .tgrp.f3.f2.b1 -text "Delete" -foreground red -activeforeground #a00000 -command {
	    if { [.tgrp.f3.f2.text curselection ] != "" } {
		.tgrp.f3.f2.text delete [.tgrp.f3.f2.text curselection ]
	    }
	}
	pack .tgrp.f3.f2.b1 -side left
	pack .tgrp.f3.f1 .tgrp.f3.f2 -side top -fill both -expand 1

	frame .tgrp.f4 -bd 2 -relief "flat"
	button .tgrp.f4.b1 -text "Apply" -foreground red -activeforeground #a00000 -command {
	    if { [.tgrp.f2.e1 get] > -2 || [.tgrp.f2.e1 get] < 65536 } {
		if { [.tgrp.f1.e1 get] != "" } {
		    set tmpline "[ .tgrp.f2.e1 get]:"
		    foreach hst [.tgrp.f3.f2.text get 0 end] {
			append tmpline "[string tolower [string trimright $hst]],"
		    }
		    set newline [string trimright $tmpline "," ] 
		    append newline ":[ .tgrp.f1.e1 get]"
		    set currentgroup [.tgrp.hidden2 get]
		    if { $currentgroup == 9999 } {
			set cmd "TAPE add group $newline"
		    } else {
			set cmd "TAPE mod group $currentgroup $newline"
		    }
		    set returnval [queryHost $currenthost $cmd 1 ]
		    if { $errno != 0 } {
			error 91
			puts stdout "ERROR:  Unable to update tape groups on $currenthost\n       Check the files permissions"
		    } else {
			repack 123
			destroy .tgrp
		    }
		}
	    }
	}
	button .tgrp.f4.b2 -text "Cancel" -foreground blue -activeforeground #000080 -command {
	    destroy .tgrp
	}
	button .tgrp.f4.b3 -text "Help" -foreground #008000 -activeforeground #004000 -command {
	    help tapegroup
	}
	pack .tgrp.f4.b1 .tgrp.f4.b2 .tgrp.f4.b3 -side left -fill x 
	pack .tgrp.f2 .tgrp.f1 .tgrp.f3 .tgrp.f4 -fill both -expand 1
	# ----- Bindings -----
	bind .tgrp.f1.e1 <Return> {
	    focus .tgrp.f2.e1
	}
	bind .tgrp.f2.e1 <Return> {
	    focus .tgrp.f3.f1.e1
	}
	bind .tgrp.f3.f1.e1 <Return> {
	    if { [.tgrp.f3.f1.e1 get] != "" } {
		.tgrp.f3.f2.text insert end [.tgrp.f3.f1.e1 get]
	    }
	    .tgrp.f3.f1.e1 delete 0 end
	}
    } else {
	.tgrp.hidden1 delete 0 end
	.tgrp.hidden2 delete 0 end
	.tgrp.hidden1 insert end $currenthost
	.tgrp.hidden2 insert end $currentgroup
	wm title .tgrp "Groups Editor: $currenthost"
    }

    # Fill boxes with data
    if { $currentgroup != 9999 } {
	.tgrp.f1.e1 delete 0 end
	.tgrp.f2.e1 delete 0 end
	.tgrp.f3.f2.text delete 0 end
	# Will have to go and get the group info from the machine
	set rdata [queryHost $currenthost "TAPE get groups" 0 ]
	foreach l $rdata {
	    if { $currentgroup == [lindex [split $l ":"] 0] } {
		.tgrp.f1.e1 insert end [lindex [split $l ":"] 2]
		.tgrp.f2.e1 insert end [lindex [split $l ":"] 0]
		set tmp [lindex [split $l ":"] 1]
		foreach user [split $tmp ","] {
		    .tgrp.f3.f2.text insert end $user
		}
	    }
	}
    }
}


#
# tapeDevs - Add and delete tape devices.
#            Set drivenum to 9999 for new drives
#
proc tapeDevs { host drivenum } {
    global errno
    if { [winfo exists .bdev ] == 0 } {
	toplevel .bdev -class Dialog
	wm title .bdev "Backup Device Editor: $host"
	wm iconname .bdev "Backup Devices"
	entry .bdev.hidden1
	entry .bdev.hidden2
	# NOT GOOD
	.bdev.hidden1 insert end $host
	.bdev.hidden2 insert end $drivenum

	frame .bdev.f1 -bd 2 -relief "raised"
	label .bdev.f1.l1 -text "ID # :" -width 7
	entry .bdev.f1.e1 -width 4
	pack .bdev.f1.l1 .bdev.f1.e1 -fill x -side left

	frame .bdev.f2 -bd 2 -relief "raised"
	label .bdev.f2.l1 -text "Name:" -width 6
	entry .bdev.f2.e1 -width 20
	pack .bdev.f2.l1 .bdev.f2.e1 -fill x -side left -expand 1

	frame .bdev.f3 -bd 2 -relief "raised"
	label .bdev.f3.l1 -text "Path:" -width 6
	entry .bdev.f3.e1 -width 20
	pack .bdev.f3.l1 .bdev.f3.e1 -fill x -side left -expand 1

	frame .bdev.f4 -bd 2 -relief "raised"
	frame .bdev.f4.f1 -bd 2 -relief "raised"
	label .bdev.f4.f1.l1 -text "Picker Path:" -width 13
	entry .bdev.f4.f1.e1 -width 20
	pack .bdev.f4.f1.l1 .bdev.f4.f1.e1 -fill x -side left -expand 1

	frame .bdev.f4.f2 -bd 2 -relief "raised"
	label .bdev.f4.f2.l1 -text "Number of slots:" -width 17
	entry .bdev.f4.f2.e1 -width 4
	pack .bdev.f4.f2.l1 .bdev.f4.f2.e1 -fill x -side left

	pack .bdev.f4.f1 .bdev.f4.f2 -side top -fill x -expand 1

	frame .bdev.f8 -bd 2 -relief "flat"
	button .bdev.f8.b1 -text "Apply" -foreground red -activeforeground #a00000 -command {
	    if { [.bdev.f1.e1 get ] > 0 || [.bdev.f1.e1 get] < 999 } {
		if { [.bdev.f3.e1 get] != "" } {
		    set newline "[.bdev.f1.e1 get ]:[.bdev.f2.e1 get ]:[.bdev.f3.e1 get ]:[.bdev.f4.f1.e1 get ]:[.bdev.f4.f2.e1 get ]"
		    set drive [.bdev.hidden2 get]
		    if { [ .bdev.hidden2 get ] == 9999 } {
			set cmd "TAPE add drive $newline"
		    } else {
			set cmd "TAPE mod drive $drive $newline"
		    }
		    set returnval [queryHost [.bdev.hidden1 get] $cmd 1 ]
		    if { $errno != 0 } {
			error 502
			puts stdout "ERROR:  Can't add backup device or file.\n       Check the files permissions"
		    } else {
			repack 126
			destroy .bdev
		    }
		} else {
		    puts stdout "ERROR:  Must provide a path for the backup file or device"
		    error 503
		}
	    } else {
		puts stdout "Pick a reasonable number like 2 or 6"
		error 504
	    }
	}
	button .bdev.f8.b2 -text "Cancel" -foreground blue -activeforeground #000080 -command {
	    destroy .bdev
	}
	button .bdev.f8.b3 -text "Help" -foreground #008000 -activeforeground #004000 -command {
	    help tapedevice
	}
	pack .bdev.f8.b1 .bdev.f8.b2 .bdev.f8.b3 -side left -fill x 


	pack .bdev.f1 .bdev.f2 .bdev.f3 .bdev.f4 .bdev.f8 -side top -fill x -expand 1
    } else {
	# Window exists
	.bdev.hidden1 delete 0 end
	.bdev.hidden2 delete 0 end
	.bdev.hidden1 insert end $host
	.bdev.hidden2 insert end $drivenum
    }
    
    # --------  Populate the widgets  --------------
    .bdev.f1.e1 configure -state normal
    .bdev.f1.e1 delete 0 end
    .bdev.f2.e1 delete 0 end
    .bdev.f3.e1 delete 0 end
    .bdev.f4.f1.e1 delete 0 end
    .bdev.f4.f2.e1 configure -state normal
    .bdev.f4.f2.e1 delete 0 end
    if { $drivenum != "9999" } {
	# Get the tape info
	set rdata [queryHost $host "TAPE get drives" 0 ]
	foreach l $rdata {
	    set val [lindex [split $l ":"] 0]
	    if { $val == $drivenum } {
		# This is the backup device to work with
		.bdev.f1.e1 insert end [lindex [split $l ":"] 0]
		.bdev.f2.e1 insert end [lindex [split $l ":"] 1]
		.bdev.f3.e1 insert end [lindex [split $l ":"] 2]
		.bdev.f4.f1.e1 insert end [lindex [split $l ":"] 3]
		.bdev.f4.f2.e1 insert end [lindex [split $l ":"] 4]
	    }
	}
	.bdev.f1.e1 configure -state disabled
	.bdev.f4.f2.e1 configure -state disabled
    }
}


#   FillTapeList  - Fill a listbox with all the hosts where MAT is installed.  
#                   Return the selected host
#                      p = parent window
#                      chost = currenthost
#                      ent  = entry widget to operate on
proc FillTapeList {p chost ent } {
    $p config -cursor watch
    global active_hosts

    set w .hList
    if ![winfo exists $w] {
        toplevel $w 
        wm withdraw $w
        frame $w.f1
        scrollbar $w.f1.vscroll -orient vertical -command "$w.f1.text yview" 
        listbox $w.f1.text -selectforeground white -selectbackground #000080 -yscrollcommand "$w.f1.vscroll set " -height 10 -width 20
        pack $w.f1.vscroll -side left -fill y
        pack $w.f1.text -side left -expand 1 -fill both
        pack $w.f1
        entry $w.hide
    }
    $w.f1.text delete 0 end
    $w.hide delete 0 end
    $w.hide insert end $ent
    foreach line $active_hosts {
	# The line of data contains numerous fields
	set hostname [ lindex [split $line ":" ] 0 ]
	$w.f1.text insert end $hostname
    }
    set x [winfo pointerx $p ]
    set y [winfo pointery $p ]
    wm geometry $w +$x+$y
    wm deiconify $w
    raise $w
    $p config -cursor xterm
    bind $w.f1.text <Double-1> {
        set tmp [ .hList.f1.text curselection ]
        if { $tmp != {} } { 
            [.hList.hide get] delete 0 end
            [.hList.hide get] insert end [.hList.f1.text get $tmp ]
            destroy .hList
        }
    }
}


#   FillGrpList  -  Fill a listbox with all the hosts in a given group.
#                   Return the selected host
#                      p = parent window
#                      chost = currenthost
#                      grp = widget containing the group
#                      ent  = entry widget to operate on
proc FillGrpList {p chost grp ent } {
    $p config -cursor watch

    set w .gList
    if ![winfo exists $w] {
        toplevel $w 
        wm withdraw $w
        frame $w.f1
        scrollbar $w.f1.vscroll -orient vertical -command "$w.f1.text yview" 
        listbox $w.f1.text -selectforeground white -selectbackground #000080 -yscrollcommand "$w.f1.vscroll set " -height 10 -width 20
        pack $w.f1.vscroll -side left -fill y
        pack $w.f1.text -side left -expand 1 -fill both
        pack $w.f1
        entry $w.hide
    }
    $w.f1.text delete 0 end
    $w.hide delete 0 end
    $w.hide insert end $ent
    
    set rdata [queryHost $chost "TAPE get groups" 0 ]
    foreach l $rdata {
	set val [lindex [split $l ":"] 0]
	if { $val == [ $grp get ] } {
	    # This is the group list to work with
	    set tmp [split [lindex [split $l ":"] 1] "," ]
	    foreach h $tmp {
		$w.f1.text insert end $h
	    }
	}
    }

    set x [winfo pointerx $p ]
    set y [winfo pointery $p ]
    wm geometry $w +$x+$y
    wm deiconify $w
    raise $w
    $p config -cursor xterm
    bind $w.f1.text <Double-1> {
        set tmp [ .gList.f1.text curselection ]
        if { $tmp != {} } { 
            [.gList.hide get] delete 0 end
            [.gList.hide get] insert end [.gList.f1.text get $tmp ]
            destroy .gList
        }
    }
}


#
# editTape - Add, delete, and modify backup media properties.
#            Set drivenum to 9999 for new drives
#
proc editTape { host tapeid } {
    global errno
    if { [winfo exists .etp ] == 0 } {
	toplevel .etp -class Dialog
	wm title .etp "Backup Media Editor: $host"
	wm iconname .etp "Backup Media"
	entry .etp.hidden1
	entry .etp.hidden2
	# NOT GOOD
	.etp.hidden1 insert end $host
	.etp.hidden2 insert end $tapeid

	frame .etp.f1 -bd 2 -relief "raised"
	label .etp.f1.l1 -text "ID # :" -width 7
	entry .etp.f1.e1 -width 4 -relief "flat"
	pack .etp.f1.l1 .etp.f1.e1 -fill x -side left

	frame .etp.f2 -bd 2 -relief "raised"
	label .etp.f2.l1 -text "Name:" -width 6
	entry .etp.f2.e1 -width 20
	pack .etp.f2.l1 .etp.f2.e1 -fill x -side left -expand 1

	frame .etp.f3 -bd 2 -relief "raised"
	label .etp.f3.l1 -text "Backup Pool#:" -width 14
	entry .etp.f3.e1 -width 4
	pack .etp.f3.l1 .etp.f3.e1 -fill x -side left -expand 1

	frame .etp.f4 -bd 2 -relief "raised"
	label .etp.f4.l1 -text "Backup Device:" -width 15
	menubutton .etp.f4.b1 -text "..." -menu .etp.f4.b1.m -relief raised -width 3
	menu .etp.f4.b1.m -tearoff 0 
	set vals {}
	set rdata [queryHost [ .etp.hidden1 get ] "TAPE get drives" 0 ]
	for { set p 0 } { $p < [llength $rdata] } { incr p 1 } {
	    set tmp [lindex $rdata $p ]
	    set t1 [lindex [split [lindex $rdata $p ] {:} ] 1 ]
	    if { $t1 != {} } {
		lappend vals $t1
	    }
	}
	foreach val $vals {
	    .etp.f4.b1.m add command -label $val -command ".etp.f4.e1 configure -state normal ; .etp.f4.e1 delete 0 end ; .etp.f4.e1 insert end \"$val\" ; .etp.f4.e1 configure -state disabled "
	}
	entry .etp.f4.e1 -width 16
	pack .etp.f4.l1 .etp.f4.b1 .etp.f4.e1 -fill x -side left
	

	frame .etp.f5 -bd 2 -relief "raised"
	label .etp.f5.l2 -text "Raw Capacity (KB):" -width 20
	menubutton .etp.f5.b2 -text "..." -menu .etp.f5.b2.m -relief raised -width 3
	menu .etp.f5.b2.m -tearoff 0 
	set vals { 120 150 240 1300 2000 2300 2600 4000 7000 20000 35000 40000 }
	foreach val $vals {
	    .etp.f5.b2.m add command -label $val -command ".etp.f5.e2 delete 0 end ; .etp.f5.e2 insert end \"$val\" "
	}
	entry .etp.f5.e2 -width 7
	pack .etp.f5.l2 .etp.f5.b2 .etp.f5.e2 -fill both -side left
	

	frame .etp.f8 -bd 2 -relief "flat"
	button .etp.f8.b1 -text "Apply" -foreground red -activeforeground #a00000 -command {
	    if { [.etp.f4.e1 get] != "" } {

		# Will have to go and get the drive info from the machine
		set mdata [queryHost [ .etp.hidden1 get ] "TAPE get drives" 0 ]
		foreach m $mdata {
		    if { [ .etp.f4.e1 get ] == [lindex [split $m ":"] 1] } {
			set line "[lindex [split $m ":"] 0]:"
		    }
		}
		append line "[ .etp.f3.e1 get ]:[ .etp.f5.e2 get ]::[ .etp.f2.e1 get ]"
		puts stdout "Cmd = $line"
		
		set cmd "TAPE add tape $line"
		
		set retval [ actionItem . "Insert the media in the drive and click OK to continue." "OK Cancel" red ]
		if { $retval == 0 } {
		    set returnval [queryHost [.etp.hidden1 get] $cmd 1 ]
		    if { $errno != 0 } {
			error 502
			puts stdout "ERROR:  Can't add backup media."
		    } else {
			repack 127
			destroy .etp
		    }
		}
		
	    } else {
		puts stdout "ERROR:  Must provide a backup device"
		error 505
	    }

	}
	button .etp.f8.b2 -text "Cancel" -foreground blue -activeforeground #000080 -command {
	    destroy .etp
	}
	button .etp.f8.b3 -text "Help" -foreground #008000 -activeforeground #004000 -command {
	    help tapemedia
	}
	pack .etp.f8.b1 .etp.f8.b2 .etp.f8.b3 -side left -fill x 


	pack .etp.f1 .etp.f2 .etp.f3 .etp.f4 .etp.f5 .etp.f8 -side top -fill x -expand 1
    } else {
	# Window exists
	.etp.hidden1 delete 0 end
	.etp.hidden2 delete 0 end
	.etp.hidden1 insert end $host
	.etp.hidden2 insert end $tapeid
    }
    
    # --------  Populate the widgets  --------------
    .etp.f1.e1 configure -state normal
    .etp.f1.e1 delete 0 end
    .etp.f2.e1 delete 0 end
    .etp.f3.e1 delete 0 end
    .etp.f4.e1 delete 0 end
    .etp.f5.e2 configure -state normal
    .etp.f5.e2 delete 0 end
    if { $tapeid != "9999" } {
	# Get the tape info
	set rdata [queryHost $host "TAPE get tapes" 0 ]
	foreach l $rdata {
	    set val [lindex [split $l ":"] 0]
	    if { $val == $drivenum } {
		# This is the backup device to work with
		.etp.f1.e1 insert end [lindex [split $l ":"] 0]
		.etp.f2.e1 insert end [lindex [split $l ":"] 1]
		.etp.f3.e1 insert end [lindex [split $l ":"] 2]
		.etp.f4.e1 insert end [lindex [split $l ":"] 3]
		.etp.f5.e1 insert end [lindex [split $l ":"] 4]
	    }
	}
	.etp.f1.e1 configure -state disabled
	.etp.f5.e2 configure -state disabled
    }
}
